# Generated by Django 5.0.7 on 2024-08-03 22:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0002_alter_tagtestbase_tags"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tagtestbase",
            name="tags",
            field=models.CharField(
                blank=True,
                default="",
                help_text="This is the tag",
                max_length=255,
                null=True,
                verbose_name="tags",
            ),
        ),
    ]
