# Generated by Django 5.0.6 on 2024-07-23 07:39

import django.db.models.deletion
import tag_me.db.models.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Post",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    tag_me.db.models.fields.TagMeCharField(
                        blank=True, max_length=255, null=True
                    ),
                ),
                ("body", models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name="TagTestBase",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "tags",
                    models.CharField(
                        help_text="This is the tag",
                        max_length=50,
                        verbose_name="tags",
                    ),
                ),
                (
                    "slug",
                    models.SlugField(
                        allow_unicode=True,
                        editable=False,
                        max_length=100,
                        unique=True,
                        verbose_name="slug",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tag ABC",
                "verbose_name_plural": "Tags ABC",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TaggedFieldTestModel",
            fields=[
                (
                    "tagtestbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.tagtestbase",
                    ),
                ),
                (
                    "tagged_field_1",
                    tag_me.db.models.fields.TagMeCharField(
                        blank=True, max_length=255, null=True
                    ),
                ),
                (
                    "tagged_field_2",
                    tag_me.db.models.fields.TagMeCharField(
                        blank=True, max_length=255, null=True
                    ),
                ),
            ],
            options={
                "verbose_name": "Tagged Field Test Model",
                "verbose_name_plural": "Tagged Field Test Models",
            },
            bases=("tests.tagtestbase",),
        ),
    ]
