{% extends "base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Change Password" %}{% endblock %}

{% block content %}
    <h1>{% trans "Change Password" %}</h1>

    <form method="POST" action="{% url 'account_change_password' %}" class="password_change">
        {% csrf_token %}
        {{ form.as_p }}
        <button type="submit" name="action">{% trans "Change Password" %}</button>
        <a href="{% url 'account_reset_password' %}">{% trans "Forgot Password?" %}</a>
    </form>
{% endblock %}
